/*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2019 Alibaba Group
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package com.meiyou.titanflutter.containers;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ProgressBar;

import com.meiyou.titanflutter.TitanFlutterView;
import com.meiyou.titanflutter.TitanFlutterPlugin;
import com.meiyou.titanflutter.interfaces.IFlutterViewContainer;

import java.util.HashMap;

import io.flutter.plugin.common.PluginRegistry;

abstract public class TitanFlutterFragment extends Fragment implements IFlutterViewContainer {

    FlutterContent mContent;
    PluginRegistry mRegistry;

    boolean resumed = false;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mRegistry = TitanFlutterPlugin.containerManager().onContainerCreate(this);
        onRegisterPlugins(mRegistry);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);
        mContent = new FlutterContent(getActivity());
        return mContent;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!resumed) {
            resumed = true;
            TitanFlutterPlugin.containerManager().onContainerAppear(this);
            mContent.attachFlutterView(getBoostFlutterView());
            Log.e("TitanFlutter", "FlutterMenuFragment resume");
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (resumed) {
            resumed = false;
            mContent.snapshot();
            TitanFlutterPlugin.containerManager().onContainerDisappear(this);
            Log.e("TitanFlutter", "FlutterMenuFragment stop");
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mContent != null) {
            mContent.destroy();
        }
        TitanFlutterPlugin.containerManager().onContainerDestroy(this);
    }

    @Override
    public void onContainerShown() {
        mContent.onContainerShown();
    }

    @Override
    public void onContainerHidden() {
        mContent.onContainerHidden();
    }

    @Override
    public TitanFlutterView getBoostFlutterView() {
        return TitanFlutterPlugin.viewProvider().createFlutterView(this);
    }

    @Override
    public boolean isFinishing() {
        return getActivity().isFinishing();
    }

    protected View createSplashScreenView() {
        FrameLayout layout = new FrameLayout(getContext());
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        params.gravity = Gravity.CENTER;
        layout.addView(new ProgressBar(getContext()),params);
        return layout;
    }

    protected View createFlutterInitCoverView() {
        View initCover = new View(getActivity());
        initCover.setBackgroundColor(Color.WHITE);
        return initCover;
    }

    @Override
    public void setBoostResult(HashMap result) {
    }

    class FlutterContent extends FlutterViewStub {

        public FlutterContent(Context context) {
            super(context);
        }

        @Override
        public View createFlutterInitCoverView() {
            return TitanFlutterFragment.this.createFlutterInitCoverView();
        }

        @Override
        public TitanFlutterView getBoostFlutterView() {
            return TitanFlutterFragment.this.getBoostFlutterView();
        }

        @Override
        public View createSplashScreenView() {
            return TitanFlutterFragment.this.createSplashScreenView();
        }
    }
}
